@@ -165,6 +165,35 @@ class VariableClUpdateRepUrl(Variable):
     value = []
 
 
+class VariableClUpdateRepList(Variable):
+    """
+    Список путей до репозиториев
+    """
+    type = "list"
+    value = []
+
+
+class VariableClUpdateInnerRepList(Variable):
+    value = []
+
+    def get(self):
+        val = self.Get('cl_update_rep_list')
+        prev_host = self.Get('cl_update_rep_hosting')
+        if prev_host:
+            for dct in val:
+                for key, value in dct.items():
+                    if key == prev_host:
+                        yield value, key
+        for dct in val:
+            for key, value in dct.items():
+                if key != prev_host:
+                    yield value, key
+
+
+class VariableClUpdateRepHosting(Variable):
+    value = ''
+
+
 class VariableClUpdateLaymanStorage(Variable):
     """
     Путь к репозиториям
@@ -288,6 +317,12 @@ class VariableClUpdateBranchRep(ReadonlyVariable):
     def get(self):
         return self.Get('cl_update_rep_name')
 
+
+class VariableClUpdateBinhostSet(Variable):
+    type = "bool"
+    value = 'off'
+
+
 class VariableClUpdateBinhostsInfo(ReadonlyVariable):
     """
     Объект для получения информации о серверах обновлений
@@ -372,6 +407,7 @@ class VariableClUpdateBinhostTime(FieldValue, Variable):
     source_variable = "cl_update_binhost_data"
     column = 3
 
+
 class VariableClUpdateBinhostLevel(FieldValue, Variable):
     """
     Список имен прочих репозиториев
@@ -1622,11 +1658,23 @@ class VariableClUpdateBinhostStableOptSet(Variable):
         return self.Get('cl_update_binhost_stable_set')
 
 
+class VariableClUpdateBinhostChoice(Variable):
+    type = "choiceedit"
+    value = ''
+    opt = ["--binhost"]
+
+    def init(self):
+        self.label = _("Mirror url")
+
+    def choice(self):
+        return self.Get("cl_update_binhost_list")
+
+
 class VariableClUpdateBinhost(Variable):
     """
     Хост с бинарными обновлениями
     """
-    value = ""
+    value = ''
 
 
 class VariableClUpdateBinhostRevisionPath(Variable):
