@@ -68,7 +68,8 @@ class Wsdl(WsdlBase):
                                     normal=(
                                         'cl_update_binhost_stable_opt_set',
                                         'cl_update_binhost_recheck_set',
-                                        'cl_update_with_bdeps_opt_set'
+                                        'cl_update_with_bdeps_opt_set',
+                                        'cl_update_binhost_choice'
                                     ),
                                     expert=(
                                         'cl_update_sync_only_set',
